#include "build.h"
#include "ff.h"
#include "proto.h"
#include "diffusion.h"

main(int argc, char *argv[])
{
  int ans, i_mol, skip, i_atom, abs;

  int i_rel, j_rel, j_species;
  int i, j, k, i_atoms, i_species, i_phant, n_species, n_species_chk, 
      n_unit_cells, n_mols, n_atoms, n_mols1;
  int period_switch, vdw_switch, vdw_rad_switch, *excl_switch, init_switch,
      phant_switch, neigh_switch, opt_switch, press_switch, one_four_switch,
      nb_switch, con_switch, sim_switch, temp_switch, start_switch;

  /* Random generator */
  long i_ran, seed, idum;

  /* Array of geometrical molecular lengths. */
  double **mol_range, mol_max_length, xmax;

  /* Declare parameters used to build the initial state and box matrices*/
  int n_layers;
  double rho, aspect_xy, aspect_xz, layer_spacing;
  double **h, **h_inv, r_off, r_on, r2_on, r2_off, r_nl, r2_nl,
         volume;
  int kc;
  /* Declare and initialize force field parameters. */
  int n_mass = 0, n_lj = 0, n_e6 = 0, n_pot = 0, n_type = 0;
  mass_entry *mass = NULL;
  lj_entry *lj = NULL;
  e6_entry *e6 = NULL;
  pot_entry *pot = NULL;
  type_entry *atm_type = NULL;
  /* File names */
  char par_file[F_MAX], *mass_file, *lj_file, *e6_file, *pot_file, 
      **struct_file, *header_file, *config_file, *header_file_save, 
      *config_file_save;
  FILE *f_trajec, *f_com_pos, *f_output;
  /* Declare and initialize molecular structure parameters. */
  int *n_atoms_per_mol = NULL, *n_mols_per_species = NULL,
      *n_bonds_per_mol = NULL, **temp_atm_lab, **temp_atm_nbr,
      ***temp_atm_br, **temp_atm_type_i;
  double ***temp_atm_pos, **temp_atm_mass, **temp_atm_chg,
      **temp_atm_sigma, **temp_atm_eps, ***temp_atm_ord;
  char ***temp_atm_type;
  int **temp_bonds_1 = NULL, **temp_bonds_2 = NULL;
  double **temp_bonds_ord = NULL;
  int *n_mols_per_unit_cell, *mol_species;

  /* Declare some arrays used in set up force field */
  double *mol_mass, *mol_mass_true;
  int *mol_first_atm;

  /* Declare combination and exclusion arrays */
  int ***exclusions, **comb_pot;
  double ***comb_par;
  /* Declare arrays used in build */
  double **scaled_atom_coords, **mol_coords, **scaled_mol_coords,
      **atom_coords, **rel_atom_coords, **atom_vels, *atom_mass, 
      *sqrt_kT_over_m, *one_over_2m, **mol_vels;
  int *atom_rel, *atom_mol, *atom_type, *hot_atom;
  int hot_flag;

  /* Scan list arrays. */
  int **scan_atm_1, **scan_atm_2;

  /* Array of the chosen principal molecular axe. */
  int *principal;
  double **mol_inert_mt, ***mol_inert_axis;
  /* Monte Carlo parameters. */
  int i_cycle, n_cycles, n_trajec, n_block, n_blocks, n_inst, i_block;

/* Array for diffusion calculation */
   double **mol_coords_corr, **mol_coords_corr_0, **scaled_mol_coords_corr, delta;
   int n_msd, first_msd, i_data, water_diffusion;
   int pos_1, pos_2, record_size, i0, j0, j0max,
      i_msd, offset, n_fit, i_fit;
   double dr[NDIM], dr_p[NDIM], dum1, dum2, interval,
      *x, *y, *sig, a, b, chi2, q, siga, sigb, y_fit,
      diffus_x, sig_diffus_x,
      diffus_y, sig_diffus_y,
      diffus_z, sig_diffus_z,
      diffus_xy, sig_diffus_xy,
      diffus_xyz, sig_diffus_xyz,
      **r_displ2, **sig_r_displ2,
      *xy_displ2, *sig_xy_displ2,
      *xyz_displ2, *sig_xyz_displ2;

  double s_sep[NDIM], sep[NDIM], r2_sep, norm;
    int j1, j2, rdfType, j_mol, n;
    char *type_i, *type_j;

 /* variable for end-to-end distance and radius of gyration calculation */
  int abs_1, abs_2, nrot;
  double **gyration, *d, **v, ee_dist, g1, g2, g3, radius_gyra, radius_gyration;
  double **order_temp, *radius_gyra_block, *radius_gyration_block, *d_temp, 
          **v_temp, **d_block, *d_ave, *d_unc, radius_gyra_ave, radius_gyra_unc, 
         radius_gyration_ave, radius_gyration_unc, *ee_dist_block, 
         ee_dist_ave, ee_dist_unc, norm1, norm2, ***gyration_block, 
        **gyration_ave, **gyration_unc, dummy; 

  /* Get command line input. */
  if (argc != 2) {
    printf("Usage: %s params_file\n", argv[0]);
    exit(1);
  }
  strcpy(par_file, argv[1]);

  /* Read in input parameter file. */
  read_diffusion_params(par_file, &header_file, &config_file, 
      &mass_file, &pot_file, 
      &n_species_chk, &excl_switch, 
      &n_cycles, &n_block,  &n_trajec, &seed, &delta, &n_msd, &first_msd, 
      &water_diffusion);

  printf("header_file = %s, config_file = %s\n",header_file,
          config_file);
  printf("mass_file = %s\n",mass_file);
  printf("pot_file = %s\n",pot_file);
  printf("n_species (from params file) = %d\n", n_species_chk);
  for (i_species = 0; i_species < n_species_chk; ++i_species)
       printf("i_specie = %d, excl_switch = %d\n", i_species,
          excl_switch[i_species]);

  printf("n_cycles = %d, \n", n_cycles);
  printf("n_trajec = %d\n", n_trajec);
  printf("seed = %ld\n",seed);
  printf("delta %g n_msd = %d first_msd %d\n", delta, n_msd, first_msd);

  /* Define additional variables. */
  if (n_cycles % n_block != 0) 
    error_exit("n_cycles must be a multiple of n_block!\n");
    n_blocks = n_cycles / n_block;

  /* Initialize random number generator. */
  i_ran = -seed;
  ran3(&i_ran);


  /* Read information about template molecules and atoms from header file */
  read_header_direct(header_file, &period_switch, &n_species,
      &n_atoms_per_mol,  &temp_atm_lab, &temp_atm_type, &temp_atm_nbr,
      &temp_atm_br, &temp_atm_ord,  &temp_atm_pos, &temp_atm_chg,
      &temp_atm_sigma, &temp_atm_eps,  &n_bonds_per_mol, &temp_bonds_1,
      &temp_bonds_2, &temp_bonds_ord,  &n_mols_per_species, &n_mols,
      &n_atoms, &mol_species);

  /* Check exclusion switch array. */
  if (n_species_chk != n_species) {
     printf("n_species_chk = %d, n_species = %d\n", n_species_chk, n_species);
     error_exit("Problem in the number of species.\n");
     }
 
  /* If the number of exclusion sites is greater than the number of atoms
     present in the template molecule, set the exclusion switch to -1, i.e.
     remove all intramolecular interactions. */ 
  for (i_species = 0; i_species < n_species; ++i_species)
    if (excl_switch[i_species] >= n_atoms_per_mol[i_species])
      excl_switch[i_species] = -1;
 
  /* Read FF masses file. */
  read_mass_params(mass_file, &mass, &n_mass);

  /* Set up force field data for masses. */
  set_up_force_field_pair(n_species, n_mols, n_atoms_per_mol, n_mass,
      mass, mol_species, n_bonds_per_mol, &temp_atm_mass, temp_atm_type,
      &mol_mass, &mol_mass_true, &mol_first_atm);

  /* Convert string tags to integer identifiers. */
  /* Create an integer identifier corresponding to template atom type. */ 
  convert_type(n_species, n_atoms_per_mol, temp_atm_type, &temp_atm_type_i, 
     &atm_type, &n_type);

  /* Set up scan list. */
  set_up_scan_list(n_species, n_atoms_per_mol, temp_atm_nbr, temp_atm_br,  
     &scan_atm_1, &scan_atm_2);


  /* Allocate memory for arrays used in the builder. */
  allocate_memory_diffusion(n_atoms, n_mols, n_species, 
      period_switch,  &scaled_atom_coords, &atom_rel, &atom_mol,
      &atom_mass, &atom_type, 
      &mol_coords, &scaled_mol_coords, 
      &rel_atom_coords, &h_inv, n_atoms_per_mol, &exclusions,
      &mol_inert_mt, &mol_inert_axis);


  /* Set up array of relative atom indices */
  relative_atoms_pair(n_mols, mol_species, mol_first_atm, n_atoms_per_mol,
      temp_atm_mass, temp_atm_type_i, atom_rel, atom_mol, atom_mass, atom_type);
      

  /* Set up the nonbonded exclusion array */
  exclusion_array(n_species, n_atoms_per_mol, excl_switch, temp_atm_nbr,  
    temp_atm_br, &exclusions);
 
   /* No of chromonic in the mixture */
  n_mols1 = n_mols_per_species[0];

   /* Allocate memory for and zero center of mass position and mean
      squared center of mass displacement arrays. */

       mol_coords_corr = allocate_2d_array(n_mols, 3, sizeof(double));
       mol_coords_corr_0 = allocate_2d_array(n_mols, 3, sizeof(double));
       r_displ2         = allocate_2d_array(n_msd+1, 3, sizeof(double));
       sig_r_displ2 = allocate_2d_array(n_msd+1, 3, sizeof(double));
       xy_displ2 = allocate_1d_array(n_msd+1, sizeof(double));
       sig_xy_displ2 = allocate_1d_array(n_msd+1, sizeof(double));
       xyz_displ2 = allocate_1d_array(n_msd+1, sizeof(double));
       sig_xyz_displ2 = allocate_1d_array(n_msd+1, sizeof(double));
       scaled_mol_coords_corr = allocate_2d_array(n_mols, 3, sizeof(double));
       gyration = allocate_2d_array(3, 3, sizeof(double));
       d = allocate_1d_array(3, sizeof(double));

   /* Allocate memory for temporary arrays. */
   order_temp = dmatrix(1, 3, 1, 3);
   d_temp = dvector(1, 3);
   v_temp = dmatrix(1, 3, 1, 3);

  
  /* Open trajectory files. */
   f_trajec = gfopen(config_file, "r");

  /* Read header from the trajectory file. */
  fread(&n_atoms, sizeof(int), 1, f_trajec);
  fread(&n_cycles, sizeof(int), 1, f_trajec);
  fread(&n_trajec, sizeof(int), 1, f_trajec);

  pos_1 = ftell(f_trajec);

  printf("Header from %s\n", config_file);
  printf("n_atoms = %d\n", n_atoms);
  printf("n_cycles = %d\n", n_cycles);
  printf("n_trajec = %d\n", n_trajec);

  /* Compute number of instantaneous configuration in trajectory file. */
  if (n_cycles % n_trajec != 0) {
      printf("n_cycles must be a multiple of n_trajec\n");
      exit(1);
  }
  n_inst = n_cycles / n_trajec;
  n_blocks = n_inst / n_block;

  /* Allocate memory for box and atom coordinates. */
  h = allocate_2d_array(3, 3, sizeof(double));
  atom_coords = allocate_2d_array(8*n_atoms, 3, sizeof(double));

   /* Allocate memory for block averages. */
   d_block = allocate_2d_array(3, n_blocks, sizeof(double));
   gyration_block = allocate_3d_array(3, 3, n_blocks, sizeof(double));
   radius_gyration_block = allocate_1d_array(n_blocks, sizeof(double));
   radius_gyra_block = allocate_1d_array(n_blocks, sizeof(double));
   ee_dist_block = allocate_1d_array(n_blocks, sizeof(double));
   d_ave = allocate_1d_array(3, sizeof(double));
   gyration_ave = allocate_2d_array(3, 3, sizeof(double));
   d_unc = allocate_1d_array(3, sizeof(double));
   gyration_unc = allocate_2d_array(3, 3, sizeof(double));

   /* Loop over data sets. */
   for (i_data = 1; i_data <= n_inst; ++i_data) {

      /* Read in atomic positions. */
      read_positions_direct(f_trajec, h, n_atoms, atom_coords);

      /* If params.period_switch == 1, calculate quantities that depend
         on box dimensions, calculate scaled atomic coordinates, and apply
         periodic boundary conditions. */
      if (period_switch) {
      box_dimensions(h, h_inv, period_switch, 0.0, 0.0);
      scaled_atomic_coords(n_atoms, h_inv, atom_coords, scaled_atom_coords);
      periodic_boundary_conditions(n_atoms, h, scaled_atom_coords,
                                      atom_coords);
      }

      /* Calculate center of mass positions. */
      center_of_mass_positions(period_switch, n_mols, n_species,
      n_atoms_per_mol,  mol_species, atom_mass, mol_first_atm,
      atom_coords, scaled_atom_coords,  scan_atm_1, scan_atm_2,
      mol_mass, h, h_inv, mol_coords, scaled_mol_coords,  rel_atom_coords);
 

  	radius_gyration = 0;
        ee_dist = 0;
        for (i = 0; i < 3; ++i)
          for (j = 0; j < 3; ++j)
             gyration[i][j] = 0.0;
  
    /* calculate the end-to-end distance and radius of gyration of chains */

         for (i_mol = 0; i_mol < n_mols1; ++i_mol) {
             
        /* Get label of the first atom and last atom of the chain */
         
             skip = mol_first_atm [i_mol];
             i_species = mol_species[i_mol];
             
             abs_1 = skip;
             abs_2 = skip + n_atoms_per_mol[i_species] -1;

             for (k = 0; k <NDIM; ++k){
             sep[k] = rel_atom_coords[abs_2][k] - rel_atom_coords[abs_1][k];
           }
            ee_dist += SQR(sep[0]) + SQR (sep[1]) + SQR (sep[2]);
       
          for (i_atom = 0; i_atom < n_atoms_per_mol[i_species]; ++i_atom) {
          abs = skip + i_atom;

             for (k = 0; k <NDIM; ++k){
             s_sep[k] = scaled_atom_coords[abs][k] - scaled_mol_coords[i_mol][k];
             s_sep[k] -= NINT(s_sep[k]);
             sep[k] = s_sep[k] * h[k][k];
           }

         radius_gyration += SQR(sep[0]) + SQR (sep[1]) + SQR (sep[2]);
         gyration[0][0] += (SQR(rel_atom_coords[abs][0]));
         gyration[1][1] += (SQR(rel_atom_coords[abs][1])); 
         gyration[2][2] += (SQR(rel_atom_coords[abs][2]));
         gyration[0][1] += rel_atom_coords[abs][0] * rel_atom_coords[abs][1];
         gyration[0][2] += rel_atom_coords[abs][0] * rel_atom_coords[abs][2];
         gyration[1][2] += rel_atom_coords[abs][1] * rel_atom_coords[abs][2];

         }
      gyration[1][0] = gyration[0][1];
      gyration[2][0] = gyration[0][2];
      gyration[2][1] = gyration[1][2];


      }
      norm1 = 1.0/(n_mols1 * n_atoms_per_mol[i_species]);
      norm2 = 1.0/n_mols1 ;

           /* Calculate block index. */
      i_block = (i_data - 1) / n_block;

      /* Add contributions to block accumulators. */
         for (i = 0; i < 3; ++i)
           for (j = 0; j < 3; ++j) {
             gyration_block[i][j][i_block] +=  gyration[i][j];
      }
      ee_dist_block[i_block] += ee_dist;
      radius_gyration_block[i_block] += radius_gyration;

      if (i_data % n_block == 0) {

         /* Calculate block averages. */
         norm = 1.0 / (n_block);
         for (i = 0; i < 3; ++i){
           for (j = 0; j < 3; ++j){
              gyration_block[i][j][i_block] *= norm * norm1;
              order_temp[i+1][j+1] =  gyration_block[i][j][i_block];
             }
          }

       ee_dist_block[i_block] *=norm * norm2;
       radius_gyration_block[i_block] *=norm * norm1;

   /* Sum of the three eigenvalues g1, g2, g3  of the gyration tensor 
      is the mean-squre radius gyration <S^2>.  Their ratios indcate the 
      shape of the chains. If they are not 
      equal to unity it means that the distribution is non-spherical */

      /* Calculate eigenvalues and eigenvectors of inertia tensor. d[1..n]
      are the eigenvalues of gyration matrix and v[1..n][1..n] are the 
        normalized eigenvectors. */
 
        jacobi(order_temp, 3, d_temp, v_temp, &nrot);

         /* Copy scratch arrays into real arrays. */
         for (i = 0; i < 3; ++i) {
           d[i] = d_temp[i+1];
           d_block[i][i_block] = d[i];
         
           radius_gyra_block[i_block] += d_block[i][i_block];
         }


         /* Print block averages to standard output. */
         printf("\nBlock averages for block %d, MC cycles %d - %d\n", i_block,
            i_block * n_block + 1, (i_block + 1) * n_block);

       for (i = 0; i < 3; ++i)
       printf("   d[%d] = %g\n", i, d_block[i][i_block]);

       printf(" Radius of gyration from eigen value %g\n", radius_gyra_block[i_block]); 
       printf(" Radius of gyration %g\n", radius_gyration_block[i_block]); 
       printf(" end-to-end distance %g\n", ee_dist_block[i_block]); 

       } /* End of Block */

   } /* Loop over all data sets finished */
 
   /* Close input and output files. */
   fclose(f_trajec);

/* Calculate the averages and uncertainties for different quantities */
   for (i = 0; i < 3; ++i)
     for (j = 0; j < 3; ++j)
        statistics(n_blocks, gyration_block[i][j], &gyration_ave[i][j],
                   &gyration_unc[i][j]);

   for (i = 0; i < 3; ++i)
     for (j = 0; j < 3; ++j)
        order_temp[i+1][j+1] = gyration_ave[i][j];
        jacobi(order_temp, 3, d_temp, v_temp, &nrot);

      /* Copy scratch arrays into real arrays. */
    for (i = 0; i < 3; ++i)
         d_ave[i] = d_temp[i+1];

   for (i = 0; i < 3; ++i)
      statistics(n_blocks, d_block[i], &dummy, &d_unc[i]);

 statistics(n_blocks, radius_gyration_block, &radius_gyration_ave,
                   &radius_gyration_unc);
 statistics(n_blocks, radius_gyra_block, &radius_gyra_ave, &radius_gyra_unc);
 statistics(n_blocks, ee_dist_block, &ee_dist_ave, &ee_dist_unc);



   /* Print run averages and uncertainties to standard output. */
   printf("\nRun averages\n");

   for (i = 0; i < 3; ++i)
      printf("   d[%d] = %g +/- %g\n", i, d_ave[i], d_unc[i]);

   printf("\n");

   printf("    end-to-end dist = %g +/- %g\n", ee_dist_ave, ee_dist_unc);
   printf("    radius_gyration = %g +/- %g\n", radius_gyration_ave, 
                            radius_gyration_unc);
   printf("    radius_gyration for eigenvalues= %g +/- %g\n", 
                     radius_gyra_ave, radius_gyra_unc);

   printf(" g1/g2 = %g, g2/g3 = %g, g1/g3 = %g\n", 
                 d_ave[0]/d_ave[1], d_ave[1]/d_ave[2], d_ave[0]/d_ave[2]);


}  /* end of main. */

/****************************************************************************/
/* Allocate memory for static arrays */
void allocate_memory_diffusion(int n_atoms, int n_mols, int n_species,
int period_switch, double ***scaled_atom_coords, int **atom_rel,
int **atom_mol, double **atom_mass, int **atom_type, 
double ***mol_coords, 
double ***scaled_mol_coords, double ***rel_atom_coords, 
double ***h_inv, int *n_atoms_per_mol,int ****exclusions, 
double ***mol_inert_mt, double ****mol_inert_axis)

{
  int i, i_rel, i_species, n_atoms_max;

  /* Allocate memory for arrays of atomic properties. */
  if (period_switch)
    *scaled_atom_coords = allocate_2d_array(n_atoms,
        3, sizeof(double));
  *rel_atom_coords = allocate_2d_array(n_atoms, 3, sizeof(double));

  /* Allocate memory for arrays of atomic properties. */
  *atom_rel = allocate_1d_array(n_atoms, sizeof(int));
  *atom_mol = allocate_1d_array(n_atoms, sizeof(int));
  *atom_mass = allocate_1d_array(n_atoms, sizeof(double));
  *atom_type = allocate_1d_array(n_atoms, sizeof(int));

  /* Allocate memory for arrays of molecular properties. */
  *mol_coords = allocate_2d_array(n_mols, 3, sizeof(double));
  if (period_switch)
    *scaled_mol_coords = allocate_2d_array(n_mols, 3,
        sizeof(double));

  /* Allocate memory for the box matrices */
  /* For now, orthorombic box */
  *h_inv = allocate_2d_array(3, 3, sizeof(double));

  /* Allocate memory for exclusion array. */
  *exclusions = allocate_1d_array(n_species, sizeof(int**));
  for (i_species = 0; i_species < n_species; ++i_species) {
    (*exclusions)[i_species] = allocate_1d_array(n_atoms_per_mol[i_species],
        sizeof(int*));
    for (i_rel = 0; i_rel < n_atoms_per_mol[i_species];
        ++i_rel) {
      (*exclusions)[i_species][i_rel] = allocate_1d_array(n_atoms_per_mol[i_species],
          sizeof(int));
    }
  }
   *mol_inert_mt = allocate_2d_array(n_mols, 3, sizeof(double));
   *mol_inert_axis = allocate_3d_array(n_mols, 3, 3, sizeof(double));
}

/****************************************************************************/
/* Set up array of relative atom indices. */
void relative_atoms_pair(int n_mols, int *mol_species, int *mol_first_atm,
   int *n_atoms_per_mol, double **temp_atm_mass, int **temp_atm_type_i,
   int *atom_rel, int *atom_mol, double *atom_mass, int *atom_type)
{
  int i_species, skip, i, i_mol, i_rel;

  /* Set up array of atomic properties. */
  for (i_mol = 0; i_mol < n_mols; ++i_mol) {
    i_species = mol_species[i_mol];
    skip = mol_first_atm[i_mol];
    for (i_rel = 0; i_rel < n_atoms_per_mol[i_species]; ++i_rel) {
      i = skip + i_rel;
      atom_rel[i] = i_rel;
      atom_mol[i] = i_mol;
      atom_mass[i] = temp_atm_mass[i_species][i_rel];
      atom_type[i] = temp_atm_type_i[i_species][i_rel];
    }
  }

}
